**CONCEPT**

If your app that will be containerized by a Dockerfile is relying on environmental variables at an .env file, the best practice is to NOT copy .env file into the image during the build phase. You would include ".env" in .dockerignore. The actual practice is to pass the env file at runtime (when starting a container off an image):  
```
docker run --env-file .env my-image
```

The reason why is that the idea of the image is that it has to do all with the app and NOT to do with the user's configurations or environmental variables. Also, this practice limits accidentally sharing images that contain the .env file (because you could push up images to image hubs).

**Issue:** What if you want to have multiple .env files simultaneously fed into `docker run`? 
Docker does not natively support multiple --env-file arguments. You could create a Makefile script (Refer to [[Makefile - PRIMER]]) that will concatenate env files beforehand:
```
cat .env1 .env2 > .env
docker run --env-file .env my-image
```
- Or you could use Docker Compose (command `docker compose`) which allows multiple env files and is generally more flexible than Docker (command `docker`). We will be covering Docker Compose at higher levels.

---

**PRACTICE**

Create .env:
```
test=123
```

Create app.js:
```
require('dotenv').config();

console.log('Hello, I can read .env file. If I had defined an env variable "test", then its value I read is:', process.env.test);
```

Create package.json:
```
{
  "dependencies": {
    "dotenv": "16.4.7"
  }
}
```

Dockerfile:
```
# Use the specific NodeJS base image (bundled in Debian)
FROM node:18

# Set the working directory in the container.
# All below commands will work from working directory.
WORKDIR /app

# Copy the package.json file to the container
COPY package.json .

# Install the NodeJS packages
RUN npm install

# Copy your application files (if any)
COPY . .

# Specify the command to run your application (optional, depends on your use case)
CMD ["node", "app.js"]
```

^ We shouldn't have to explain any of the above lines because you would've mastered them by this point in the challenge series.

.dockerignore:
```
.DS_Store
node_modules
package-lock.json
__pycache__
*.pyc
.env
```

Let's build the image (Again, Docker places the image file somewhere on our system):
```
docker build -t ev_app .
```

Then let's run a virtual container off the image while passing environmental variables file .env:
```
docker run --env-file .env ev_app
```

Expect output to look like:
```
Hello, I can read .env file. If I had defined an env variable "test", then its value I read is: 123
```

Test on remote system:
Copy your folder of files into a remote server. Then SSH and repeat the steps to check it's the same output when you build then run the image.

Cleanup:
Run `docker images` to see the image we've created performing this challenge. Then run this command to delete the image: `docker rmi NAME:TAG --force`

